//
//  HomematicLogicalLayer.js
//  Homematic Virtual Interface Core
//
//  Created by Thomas Kluge on 20.11.16.
//  Copyright © 2016 kSquare.de. All rights reserved.
//


"use strict";

var path = require('path')
var Url = require("url");
var fs = require('fs');
const util = require('util');

var xmlrpc = require(path.join(__dirname,  "/homematic-xmlrpc"));
var request = require("request");
var logger = require(path.join(__dirname,  "/logger.js")).logger("HomematicLogicalLayer");
var HomematicChannel = require(path.join(__dirname,  "/HomematicChannel.js")).HomematicChannel;
var HomematicDevice = require(path.join(__dirname,  "/HomematicDevice.js")).HomematicDevice;
var regaRequest = require(path.join(__dirname,  "/HomematicReqaRequest.js"));
const EventEmitter = require('events');

var RPCConsumer = function() {

}

RPCConsumer.prototype.methodCall = function (method, params, callback) {
	var that = this;
	logger.debug("RPC Out -> %s %s",method,JSON.stringify(params));
	this.client.methodCall(method , params, function(error, value) {
		callback(error,value)
	});
}


RPCConsumer.prototype.encode = function() {
	return {"hostname":this.hostname,"port":this.port,"path":this.path,"callbackid":this.callbackid};
}

RPCConsumer.prototype.initwithurl = function(ccuip,urlstring,callbackid) {

 // replace xmlrpc_bin 
 
  urlstring = urlstring.replace("xmlrpc_bin", "http");
  urlstring = urlstring.replace("xmlrpc", "http");
 
  var purl = Url.parse(urlstring);
  this.port = purl.port; 
  this.path = purl.pathname;
  this.callbackid = callbackid;
  this.hostname = "";
  
  switch (purl.hostname) {
	  
	  case "127.0.0.1":
	  this.hostname = ccuip;
	  break;
	  case undefined:
	  this.hostname = ccuip;
	  break;
	  default:
	  this.hostname = purl.hostname;
  }
  
  
  this.client = xmlrpc.createClient({
			host: this.hostname,
			port: this.port,
			path: this.path
  });
}

RPCConsumer.prototype.initwithcoder = function(encodedobject,ccuip) {


  this.port = encodedobject.port; 
  this.path = encodedobject.path;
  this.callbackid = encodedobject.callbackid;
  
  if (encodedobject.hostname == undefined) {
	  encodedobject.hostname = "127.0.0.1";
  }
  
  if (encodedobject.hostname == "127.0.0.1") {
	  this.hostname = ccuip;
  } else {
	  this.hostname = encodedobject.hostname;
  }

	this.client = xmlrpc.createClient({
			host: this.hostname,
			port: this.port,
			path: this.path
  	});

}

RPCConsumer.prototype.description = function() {
  return this.hostname + ":" + this.port + this.path;
}

RPCConsumer.prototype.isEqual = function(rpcconsumer) {

  var result = true;
  
  if (this.port != rpcconsumer.port) {
	  result = false;}
  if (this.hostname != rpcconsumer.hostname) {
	  result = false;}
  if (this.path != rpcconsumer.path) {
	  result = false;}
  return result;  
}


var HomematicLogicalLayer = function (config) {
	this.rpc_server;
	this.clients = [];
	this.devices = [];
	this.rpcClient;
	this.interfaceID;
	this.interfaceCallbackId;
	this.config = config;
	this.consumer = [];
	this.homematicChannel = HomematicChannel;
	this.homematicDevice = HomematicDevice;
	this.lastMessage = "";
	this.rpcClients = {} ;
	this.rpcEventServer = undefined ;
	this.CCUEventcache = {};
	
	this.ccuIP = this.config.getValue("ccu_ip");
	if (this.ccuIP == undefined) {
		logger.error("please setup your ccu ip in config.json");
	} else {
	    logger.info("welcome. will create a interface layer for CCU at " + this.ccuIP);
		this.loadConsumer(this.ccuIP);
	}
}

util.inherits(HomematicLogicalLayer, EventEmitter);

HomematicLogicalLayer.prototype.init = function() {
    var that = this;
	var localPort = 7000;
    var localip = this.config.getValue("local_ip");
    

    if (localip == undefined) {
		 localip = this.getIPAddress();
    } 
	if (localip == "0.0.0.0") {
      logger.error("Cannot fetch my own ip");
	}

    logger.info("MyIP is " + localip);
	this.localip = localip
	
	try {
		
	this.rpc_server = xmlrpc.createServer({
      host: localip,
      port: localPort
	});

	} catch (e) {
		logger.error ("Cant start rpc");
	}


	this.rpc_server.on("NotFound", function(method, params) {
      logger.debug("Method " + method + " does not exist. - " + JSON.stringify(params));
	});

	logger.debug("XML-RPC server for interface Homematic Virtual Interface is listening on port " + localPort);


 	this.methods = {
   	'system.listMethods': function listMethods(err, params, callback) {
	   		that.lastMessage = new Date();
	   	    logger.debug('rpc < system.listMethods', null, params);
            logger.debug('repl  >', null, JSON.stringify(Object.keys(that.methods)));
            callback(null,Object.keys(that.methods));
    },
    
    'listDevices': function listDevices(err, params, callback) {
	    	
	   		that.lastMessage = new Date();
       		logger.debug('rpc < listDevices', null, params);
       		
       		if (err.headers["user-agent"]=="Apache-HttpClient/4.2.5 (java 1.5)") {
	       	 logger.debug("Hide Devices from HMServer")
	       	callback(null, []);
            	
       		} else {
	       		
       		
       		var devices = that.getMyDevices();
            callback(null, devices);}
            
    },
    
    'getDeviceDescription': function getDeviceDescription(err, params, callback) {
    
	   		that.lastMessage = new Date();
		    logger.debug('rpc < getDeviceDescription %s', params);
    		var adress = params[0];
			var found = false;
			
			var device = that.deviceWithAdress(adress)
			if (device) {
				logger.debug("Device Found ")
				var re = device.getDeviceDescription()
	    	    logger.debug('(D)repl  > %s', JSON.stringify(re));
	    	    found = true;
	    	    callback(null,re)
	    	    return;
			} else {
				var channel = that.channelWithAdress(adress)
				if (channel) {
					logger.debug("Channel Found ")
					var re = channel.getChannelDescription();
					logger.debug('(C)repl  > %s', JSON.stringify(re));
					found = true;
					callback(null,re)
					return;
				} else {
					logger.debug('repl > getDeviceDescription %s', "Nothing Found");
					callback({"faultCode":-2,"faultString":"Unknown instance"},undefined);
				}
			}
    },
    
    
    'getLinks': function getLinks(err, params, callback) {
    
	   		that.lastMessage = new Date();
		    logger.debug('rpc < getLinks %s', params);
		    logger.debug('repl  > %s %s', null, []);
    		callback(null,[]);
     },
    
    
    'getValue': function getValue(err, params, callback) {
	   		that.lastMessage = new Date();
    		logger.debug('rpc < getValue %s', params);
		    var adress = params[0];
			var parameterName = params[1];
			that.devices.forEach(function (device) {
	    	  	    	  
	    	  device.channels.forEach(function (channel) {
	     		if (channel.adress == adress) {
	    	    var re = channel.getParamsetValue("VALUES",parameterName);
	    	    logger.debug('repl  > %s', JSON.stringify(re));
	    	    callback(null,re)
	    	    return;
	    	  }
	   	     });
	    	  
	    	});

     },
    
    'setValue': function setValue(err, params, callback) {
	   		that.lastMessage = new Date();
    		logger.debug('rpc < setValue %s', params);
		    var adress = params[0];
			var parameterName = params[1];
			var value = params[2];
			that.devices.forEach(function (device) {
				device.channels.forEach(function (channel) {
	    	    if (channel.adress == adress) {
	    	    	channel.setValue(parameterName,value);
	    	   		logger.debug('repl  > %s %s', null, null);
	    	   		callback(null,undefined);
	    	    return;
	    	  }
	   	     });
	    	  
	    	});

     },
    

    
    'getParamsetDescription': function getParamsetDescription(err, params, callback) {
	   		that.lastMessage = new Date()
			logger.debug('rpc < getParamsetDescription %s', params)
			var adress = params[0]
			var paramset = params[1]
			var found = false
			that.devices.forEach(function (device) {
	    	  if (device.adress === adress) {
	    	    var re = device.getParamsetDescription(paramset)
	    	    logger.debug('repl  > %S', JSON.stringify(re))
	    	    callback(null,re)
	    	    found = true
	    	    return
	    	  }
	    	  
	    	  device.channels.forEach(function (channel) {
	     		if (channel.adress === adress) {
	    	    var re = channel.getParamsetDescription(paramset)
	    	    logger.debug('repl  > %s', JSON.stringify(re))
	    	    callback(null,re)
	    	    found = true
	    	    return
	    	  }
	   	     });
	    	  
	    	});

		if (found===false) {	    	
	    	logger.debug('repl > getParamsetDescription', "Nothing Found");
			callback({"faultCode":-2,"faultString":"Unknown instance"},undefined);
		}
	},

    'getParamsetId': function getParamsetId(err, params, callback) {
	   		that.lastMessage = new Date();
			logger.debug('rpc < getParamsetId %s', params);
			var adress = params[0];
			var paramset = params[1];
			that.devices.forEach(function (device) {
	    	  if (device.adress == adress) {
	    	    var re = device.getParamsetId(paramset)
	    	    logger.debug('repl  > %s', JSON.stringify(re));
	    	    callback(null,re)
	    	    return;
	    	  }
	    	  
	    	  device.channels.forEach(function (channel) {
	     		if (channel.adress == adress) {
	    	    var re = channel.getParamsetId(paramset);
	    	    logger.debug('repl  > %s', JSON.stringify(re));
	    	    callback(null,re)
	    	    return;
	    	  }
	   	     });
	    	  
	    	});
	    	

	},

   'getParamset': function getParamset(err, params, callback) {
	   		that.lastMessage = new Date();
			logger.debug('rpc < getParamset %s', params);
			var adress = params[0];
			var paramset = params[1];
			that.devices.forEach(function (device) {
	    	  if (device.adress == adress) {
	    	    var re = device.getParamset(paramset)
	    	    logger.debug('repl  > %s', JSON.stringify(re));
	    	    callback(null,re)
	    	    return;
	    	  }
	    	  
	    	  device.channels.forEach(function (channel) {
	     		if (channel.adress == adress) {
	    	    var re = channel.getParamset(paramset);
	    	    logger.debug('repl  > %s', JSON.stringify(re));
	    	    callback(null,re)
	    	    return;
	    	  }
	   	     });
	    	  
	    	});
	    	

	},



	'reportValueUsage': function reportValueUsage(err, params, callback) {
	   		that.lastMessage = new Date();
			logger.debug('rpc < reportValueUsage %s', params);
			// thank you for the fish ...
			callback(null,[]);
	},


	'deleteDevice': function deleteDevice(err, params, callback) {
	   		that.lastMessage = new Date();
			logger.debug('rpc < deleteDevice %s', params);
			var adress = params[0];
			var mode = params[1];
			logger.debug('repl  >', []);
			callback(null,[]);
			
			// call Bridge and set Flag
			var device = that.deviceWithAdress(adress);
			if (device != undefined) {
				that.deleteDevice(device,false);
			}
			// rega callback
			that.sendRPCMessage(undefined,"deleteDevices",[adress], function(error, value) {});
	},


	'getLinkPeers': function getLinkPeers(err, params, callback) {
	   		that.lastMessage = new Date();
			logger.debug('rpc < getLinkPeers %s', params);
			var adress = params[0];
			var mode = params[1];
			logger.debug('repl  >', null);
			callback(null,[]);
	},

	'system.methodHelp' : function methodHelp(err,params,callback) {
	   		that.lastMessage = new Date();
			logger.debug('rpc < methodHelp %s %s', err, params);
			logger.debug('repl  >', null);
			callback(null,[]);
	},


    'putParamset': function putParamset(err, params, callback) {
	   		that.lastMessage = new Date();
			logger.debug('rpc < putParamset %s', params);
			var adress = params[0];
			var paramset = params[1];
			var parameters = params[2];
			that.devices.forEach(function (device) {
	    	  if (device.adress == adress) {
	    	    var re = device.putParamset(paramset,parameters)
	    	    logger.debug('repl  >', null, JSON.stringify(re));
	    	    device.resetConfigPending();
				that.saveDevice(device);
	    	    callback(null,re)
	    	    return;
	    	  }
	    	  
	    	  device.channels.forEach(function (channel) {
	     		if (channel.adress == adress) {
	    	    var re = channel.putParamset(paramset,parameters);
	    	    logger.debug('repl  > %s',JSON.stringify(re));
	    	    device.resetConfigPending();
				that.saveDevice(device);
	    	    callback(null,re)
	    	    return;
	    	  }
	   	     });


	    	});
			callback(null,[]);
	},

	
    'init' : function init(err, params, callback) {
	   		that.lastMessage = new Date();
            logger.debug('rpc < init %s', params);
            var intf = params[1];
 			var cns = new RPCConsumer();
 			
 			if (intf != "") {
	 			if (that.isHMServer(intf)==false) {
 				cns.initwithurl(that.ccuIP,params[0],params[1]);
				that.addConsumer(cns);
				that.saveConsumer();
				logger.info("connection request from your ccu .. live is good");
				} else {
					logger.debug("Ignore HMServer because the front fell off");
				}
 			} else {
 				that.removeOldConsumer(cns);
 				that.saveConsumer();
				logger.info("there is a removal");
			}
	}
	
	};


    Object.keys(that.methods).forEach(function (m) {
           that.rpc_server.on(m, that.methods[m]);
    });
    
	this.rpc_server.close = function() {
		logger.info("RPC Server was removed.");
    };
}

HomematicLogicalLayer.prototype.getLocalIpAdress = function() {
 return this.localip
}

HomematicLogicalLayer.prototype.addEventNotifier = function(callback) {
  	
  if (this.rpcEventServer==undefined) {
	var that = this	
	var port = 7002
	this.rpcEventServer = xmlrpc.createServer({
    host: that.localip,
    port: port
  	});
    var methods = {
   	'system.listMethods': function listMethods(err, params, callback) {
            callback(null,Object.keys(methods));
    },
    'listDevices': function listDevices(err, params, callback) {
      callback(null,[]);
    },
    'newDevices': function newDevices(err, params, callback) {
      callback(null,[]);
    },
    'event': function event(err, params, callback) {
      callback(null,[]);
    },
    
    'system.multicall': function systemmulticall(err, params, callback) {
      params.map(function(events) {
        try {
          events.map(function(event) {
            if ((event["methodName"] == "event") && (event["params"] !== undefined)) {
              var params = event["params"];
              var intf = params[0];
              var strIf = ""
              if (intf.indexOf('hvl_')==0) {
	             strIf = intf.slice(4);
              }
              var channel = params[1];
              var datapoint = params[2];
              var value = params[3];
          	  logger.debug("RPC event (%s) for %s %s with value %s",strIf,channel,datapoint,value);
          	  that.doCache(strIf,channel,datapoint,value);
          	  try {
			  	  that.emit('ccu_datapointchange_event',strIf, channel,datapoint,value);
		  	  } catch (e){
			  	  logger.error("MultiCall Error %s",e.stack)
		  	  }
            }
          });
        } catch (err) {}
      });
      callback(null,[]);
    } 
	};
    
    Object.keys(methods).forEach(function (m) {
           that.rpcEventServer.on(m, methods[m]);
    });

    EventEmitter.call(this);

    this.addRPCClient("BidCos-RF");
    this.callRPCMethod('BidCos-RF','init', ["http://" + that.localip + ':' + port , 'hvl_BidCos-RF' ], function(error, value) {
      logger.debug("CCU Response ...Value (%s) Error : (%s)",JSON.stringify(value) , error);
	  if (callback) {
		  callback()
	  }
    });

  } else {
	  if (callback) {
		  callback()
	  }
  }
    
}

HomematicLogicalLayer.prototype.doCache = function(interf,adress,datapoint,value) {
  var adr = interf + "." + adress + "." + datapoint;
  var el = this.CCUEventcache[adr];
  if (!el) {
	  el = {"v":value,"t":Date.now()};
  } else {
	  el["l"] = el['v'];
	  el["v"] = value;
	  el["t"] = Date.now();
  }
  this.CCUEventcache[adr]=el;
}

HomematicLogicalLayer.prototype.getCachedState = function(interf,adress,datapoint) {
	var adr = interf + "." + adress + "." + datapoint;
	var dp = this.CCUEventcache[adr];
	return (dp) ? dp['v'] : undefined;
}

HomematicLogicalLayer.prototype.addRPCClient = function(intf) {
  if (this.rpcClient(intf) === undefined) {
	  
	  var port = 0
	  var path = '/'
	  var ccuIP = this.ccuIP
	  
	  switch (intf) {
		  
		  case "BidCos-RF":
		   port = 2001
		   break
		  case "BidCos-Wired":
		   port = 2000
		   break
		  case "HmIP-RF":
		   port = 2010
		   break
	  }
	  
	var client = xmlrpc.createClient({
			host: ccuIP,
			port: port,
			path: path
	})
	
	this.rpcClients[intf] = client
	return client
  } else {
	return this.rpcClient(intf)
  } 
}

HomematicLogicalLayer.prototype.callRPCMethod = function(intf,command,parameters,callback) {
	var client = this.rpcClient(intf)
	if (client) {
		try {
			client.methodCall(command,parameters,function(error, value) {
			if (callback) {
				callback(error,value)
			}
		})
		} catch (e) {
			logger.error("RPC Error %s",e)
		}
	}
}

HomematicLogicalLayer.prototype.rpcClient = function(intf) {
 return this.rpcClients[intf];
}



HomematicLogicalLayer.prototype.isHMServer = function(intf) {
	//return false;
	
	return intf.indexOf("_java", intf.length - "_java".length) !== -1;
}

HomematicLogicalLayer.prototype.addConsumer = function(aConsumer) {
  // TODO Check if we have to purge old Consumers
  logger.debug("Adding new Consumer");
  this.consumer = this.removeOldConsumer(aConsumer);
  this.consumer.push(aConsumer);
}

HomematicLogicalLayer.prototype.saveConsumer = function() {
  // Encode
  var encoded = [];
  this.consumer.map(function(consumer){
	  encoded.push(consumer.encode());
  });
  this.config.setPersistValue("consumer",JSON.stringify(encoded));
}

HomematicLogicalLayer.prototype.consumerWithID = function(cid) {

  this.consumer.forEach(function(aConsumer){
	  if (aConsumer.callbackid == cid) {
		  return aConsumer;
	  }
  });
  
  return undefined;
}


HomematicLogicalLayer.prototype.cleanUp = function() {
	this.consumer = [];	
	this.saveConsumer();
	this.lastMessage = "";
}

HomematicLogicalLayer.prototype.removeOldConsumer = function(newconsumer) {
  var result =  this.consumer.filter(function(ctt) { return !ctt.isEqual(newconsumer); });
  return result;
}

HomematicLogicalLayer.prototype.loadConsumer = function(ccuip) {
  var that = this;
  this.consumer = [];
  var myccuip = ccuip;
  
  var consumer = this.config.getPersistValue("consumer");
  try {
  if ((consumer) && (consumer!="undefined")) {
	  var oconsumer = JSON.parse(consumer);
	  if (oconsumer != undefined) {
	  
	  oconsumer.map(function(obj) {
	  		logger.info('init consumer %s',JSON.stringify(obj))
		    var cns = new RPCConsumer();
 			cns.initwithcoder(obj,myccuip);
 			that.addConsumer(cns);
	  });
	  }
  } else {
	  logger.info("No Consumer .. is this your first time ?")
  }
  
  } catch (e){
	  logger.error("Consumer Error %s",e)
  }
}

HomematicLogicalLayer.prototype.shutdown = function() {
	this.rpc_server.close();
	var port = 7002;
	this.callRPCMethod('BidCos-RF','init', ["http://" + this.localip + ':' + port , null ], function(error, value) {
      logger.debug("CCU Response ...Value (%s) Error : (%s)",JSON.stringify(value) , error);
    });
	logger.debug("Virtual Layer Shutdown");
}


HomematicLogicalLayer.prototype.sendRPCEvent = function(adress,parameters) {
   var that = this;
   
   this.consumer.forEach(function (tc){
		var eventPayload = [];
		var cid = tc.callbackid;
		 parameters.forEach(function (parameter) {
	     	if (parameter.type=="FLOAT") {
		 		eventPayload.push({"methodName":"event","params":[cid ,adress,parameter.name,{"explicitDouble":parameter.value}]})
		 	} else {
		 		eventPayload.push({"methodName":"event","params":[cid ,adress,parameter.name,parameter.value]})
		 	}
		 	tc.methodCall("system.multicall", [eventPayload] , function(error, value) {
			 
		 	});
     });
   });
}

HomematicLogicalLayer.prototype.getMyDevices = function() {
      var result = [];
      var that = this;
      this.devices.some(function (device) {
	    // Check if there is a deletion flag
	    if (!that.wasDeletedBefore(device)) {
	    	if (device.hidden == false) {
		    	result.push(device.getDeviceDescription())
				device.channels.forEach(function (channel) {
					result.push(channel.getChannelDescription())
				});	
	    	}
	    }
	  });
     return result; 
}


HomematicLogicalLayer.prototype.publishAllDevices = function(callback) {
	var that = this;
	this.sendRPCMessage(undefined,"newDevices",this.getMyDevices(), function(error, value) {
		that.devices.forEach(function (device) {
			if (device.hidden == false) {
				device.wasPublished = true;
				that.saveDevice(device);
			}
		});
		callback();
	});
}

HomematicLogicalLayer.prototype.sendRPCMessage = function(consumerID, method,payload,callback) {
  
  if (consumerID == undefined) {
	  // send to all
    logger.debug("Will send to all Consumers");
	this.consumer.forEach( function(aConsumer){
	    aConsumer.methodCall(method, [aConsumer.callbackid,payload] , function(error, value) {
    				logger.debug("RPC %s Response %s Errors: %s",method,JSON.stringify(payload),error);
    				if (callback!=undefined) {
    				  callback(error,value);
    				}
    	});
	});
  } else {
    var consumer = this.consumerWithID(consumerID);
    if (consumer != undefined) {
	    consumer.methodCall(method, [consumerID,payload] , function(error, value) {
    				logger.debug("RPC %s Response %s Errors: %s",method,JSON.stringify(payload),error);
    				if (callback!=undefined) {
    				  callback(error,value);
    				}
    	});
    }
 }
}

HomematicLogicalLayer.prototype.addDevice = function(device,save,hidden) {
   logger.debug("Add new Device to HomematicLogicLayer " + device.adress);
   device.hidden = hidden || false;
   this.devices.push(device);
   logger.debug("Pushed to list %s",device.hidden);
   var that = this;
   
   if (!hidden) {
   	// Add Listener to Working Events to publish
   		device.on("event_device_channel_value_change", function(parameter){
		 logger.debug("Device Value Change Event for %s",parameter.channel);
		 that.sendRPCEvent(parameter.channel,parameter.parameters);
		});
   }
   
  
   
   if ((!device.wasPublished) && (!device.hidden)) {
   		logger.debug("Send this to CCU");
   		var result = [];
   		result.push(device.getDeviceDescription())
	    
   		device.channels.forEach(function (channel) {
	    	 result.push(channel.getChannelDescription())
  		});
   
  		this.sendRPCMessage(undefined,"newDevices",result, function(error, value) {
	  		device.wasPublished = true;
	  		if (save) {that.saveDevice(device);}
    	});
    	
   } else {
	  logger.debug("CCU should (not) know about %s",device.adress);
      if (save) {this.saveDevice(device);}
   }
   
}

HomematicLogicalLayer.prototype.deviceDataWithSerial = function(serial) {
  var persistenceFile = this.config.storagePath() + "/" + serial + ".dev";
  try {
    var data = fs.readFileSync(persistenceFile);
    // Try to parse //
	    JSON.parse(data);
    return data;
  } catch (e) {
    return undefined;
  }
}

HomematicLogicalLayer.prototype.wasDeletedBefore = function(device) {
	var result = false
	var deletedDevices = this.config.getPersistValueWithDefault("deletedDevices",[]);
	if (deletedDevices) {
		deletedDevices.some(function (adress){
			if (device.adress===adress) {
				result = true;
			}
		})
	}
	
	return result
}


HomematicLogicalLayer.prototype.saveDevice = function(device) {
   var persistenceFile = this.config.storagePath() + "/" + device.serialNumber + ".dev";
   logger.debug("make device %s persistent at %s",device.serialNumber,persistenceFile);
   fs.writeFile(persistenceFile, device.savePersistent(), function(err) {});
}

HomematicLogicalLayer.prototype.removeStoredDeviceData = function(device) {
   var persistenceFile = this.config.storagePath() + "/" + device.serialNumber + ".dev";
   try {
     fs.unlinkSync(persistenceFile);
   } catch (err) {}
}


HomematicLogicalLayer.prototype.deviceWithAdress = function(adress) {
	
	var result = undefined;
	
	this.devices.forEach(function (device) {
		if (device.adress == adress) {
			result = device;
		}
		});
	return result;
}


HomematicLogicalLayer.prototype.channelWithAdress = function(adress) {
	var deviceAdress = adress.slice(0,adress.indexOf(":"));
	var selectedDevice = this.devices.filter(function (device) { return device.adress == deviceAdress}).pop();
	
	if (selectedDevice) {
		var selChannel =  selectedDevice.channels.filter(function (channel) { return channel.adress == adress}).pop();
		return selChannel;
		
	}
	return undefined;
}

HomematicLogicalLayer.prototype.devicesWithNameLike = function(name) {
	
	var result = [];
	var re = new RegExp("^"+name+"*.?");
	
	this.devices.forEach(function (device) {
		if (re.test(device.adress)) {
			result.push(device);
		}
		});
	return result;
}


HomematicLogicalLayer.prototype.deleteDevice = function(device,publish) {
   logger.debug("Remove Device from HomematicLogicLayer " + device.adress);
   var index = this.devices.indexOf(device);
   if (index > -1) {
    this.devices.splice(index, 1);
	// Remove persistence    
     var persistenceFile = this.config.storagePath() + "/" + device.serialNumber + ".dev";
     try {
	     fs.unlinkSync(persistenceFile);
		 var deltedDevices = this.config.getPersistValueWithDefault("deletedDevices",[]);
		 deltedDevices.push(device.adress);
		 this.config.setPersistValue("deletedDevices",deltedDevices);
     } catch (err) {
	     logger.error("delete device error %s",err);
     }
     // Send that to all consumers
     if (publish == true) {
	    this.sendRPCMessage(undefined,"deleteDevices",[device.adress], function(error, value) {});
     }
     
   }
}


HomematicLogicalLayer.prototype.deleteDevicesByOwner = function(device_owner) {
  var that = this;
  this.devices.some(function (device){
	if ((device.owner) && (device.owner === device_owner)) {
	   that.deleteDeviceTemporary(device);
	}
  })
}


HomematicLogicalLayer.prototype.deleteDeviceWithAdress = function(adress) {
   logger.debug("Remove Device Rega " + adress);
   this.sendRPCMessage(undefined,"deleteDevices",[adress], function(error, value) {});
}

HomematicLogicalLayer.prototype.deleteDeviceTemporary = function(device) {
   logger.debug("temporary Remove Device from HomematicLogicLayer " + device.adress);
   var index = this.devices.indexOf(device);
   if (index > -1) {
    this.devices.splice(index, 1);
   }
}

HomematicLogicalLayer.prototype.runRegaScript = function(script,callback) {
    new regaRequest(this,script,function(result){
	  if (callback) {callback(result)}
	})
}

HomematicLogicalLayer.prototype.loadCCUProgramms = function(callback) {
    var that = this;
    var result_list = {};
    var script = "string pid;boolean df = true;Write(\'{\"programs\":[\');foreach(pid, dom.GetObject(ID_PROGRAMS).EnumUsedIDs()){var prg = dom.GetObject(pid);if(df) {df = false;} else { Write(\',\');}Write(\'{\');Write(\'\"id\": \"\' # pid # \'\",\"name\": \"\' # prg.Name() # \'\"}\');}Write(\"]}\");\";"
    
    new regaRequest(this,script,function(result){
	    
	    try {
		    if (result) {
		    	var jobj = JSON.parse(result);
				callback(jobj);
			}  
	    } catch (e) {
		    console.log(e.stack);
	    }
    });
}

HomematicLogicalLayer.prototype.loadCCUVariables = function(callback) {
    var that = this;
    var result_list = {};
    var script = "string pid;boolean df = true;Write(\'{\"variables\":[\');foreach(pid, dom.GetObject(ID_SYSTEM_VARIABLES).EnumUsedIDs()){var vrb = dom.GetObject(pid);if(df) {df = false;} else { Write(\',\');}Write(\'{\');Write(\'\"id\": \"\' # pid # \'\",\"name\": \"\' # vrb.Name() # \'\" ,\"type\": \"\' # vrb.ValueType() # \'\" ,\"subtype\": \"\' # vrb.ValueSubType() # \'\"    }\');} Write(\"]}\");\";"
    
    new regaRequest(this,script,function(result){
	    
	    try {
		    if (result) {
		    	var jobj = JSON.parse(result);
				callback(jobj);
			}  
	    } catch (e) {
		    console.log(e.stack);
	    }
    });
}

HomematicLogicalLayer.prototype.loadCCUDevices = function(interfaces, callback) {
    var that = this;
    
    var ifSelector = "(oInterface.Name() == 'BidCos-RF')";
    
    if (interfaces.indexOf('BidCos-Wired')>-1) {
	    ifSelector = ifSelector + " || (oInterface.Name() == 'BidCos-Wired')"
    }

    if (interfaces.indexOf('HmIP-RF')>-1) {
	    ifSelector = ifSelector + " || (oInterface.Name() == 'HmIP-RF')"
    }
    
    
    var script = "string sDeviceId;string sChannelId;boolean df = true;Write(\'{\"devices\":[\');foreach(sDeviceId, root.Devices().EnumIDs()){object oDevice = dom.GetObject(sDeviceId);if(oDevice){var oInterface = dom.GetObject(oDevice.Interface());if (" + ifSelector + ") { if(df) {df = false;} else { Write(\',\');}Write(\'{\');Write(\'\"id\": \"\' # sDeviceId # \'\",\');Write(\'\"if\": \"\' # oInterface # \'\",\');Write(\'\"name\": \"\' # oDevice.Name() # \'\",\');Write(\'\"address\": \"\' # oDevice.Address() # \'\",\');Write(\'\"type\": \"\' # oDevice.HssType() # \'\",\');Write(\'\"channels\": [\');boolean bcf = true;foreach(sChannelId, oDevice.Channels().EnumIDs()){object oChannel = dom.GetObject(sChannelId);if(bcf) {bcf = false;} else {Write(\',\');}Write(\'{\');Write(\'\"cId\": \' # sChannelId # \',\');Write(\'\"name\": \"\' # oChannel.Name() # \'\",\');if(oInterface){Write(\'\"intf\": \"\' # oInterface.Name() 	# \'\",\');Write(\'\"address\": \"\' # oInterface.Name() #\'.'\ # oChannel.Address() # \'\",\');}Write(\'\"type\": \"\' # oChannel.HssType() # \'\"\');Write(\'}\');}Write(\']}\');}}}Write(\']}\');";
    
    new regaRequest(this,script,function(result){
	    
	    try {
		    if (result) {
		    	var jobj = JSON.parse(result);
				callback(jobj);
			}
		} catch (error) {
			
		}
    });
}



HomematicLogicalLayer.prototype.getIPAddress = function() {
    var interfaces = require("os").networkInterfaces();
    for (var devName in interfaces) {
      var iface = interfaces[devName];
      for (var i = 0; i < iface.length; i++) {
        var alias = iface[i];
        if (alias.family === "IPv4" && alias.address !== "127.0.0.1" && !alias.internal)
          return alias.address;
      }
    }
    return "0.0.0.0";
}

HomematicLogicalLayer.prototype.listConsumer = function() {
 return this.consumer;
}

module.exports = {
  HomematicLogicalLayer : HomematicLogicalLayer
}
